@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import collections
 import dataclasses
 import datetime
 import functools
@@ -9,6 +10,8 @@ import weakref
 from typing import *
 from weakref import ref
 
+from outleap import LEAPClient
+
 from hippolyzer.lib.base.datatypes import UUID
 from hippolyzer.lib.base.helpers import proxify
 from hippolyzer.lib.base.message.message import Message
@@ -41,7 +44,8 @@ class Session(BaseClientSession):
         self.circuit_code = circuit_code
         self.global_caps = {}
         # Bag of arbitrary data addons can use to persist data across addon reloads
-        self.addon_ctx = {}
+        # Each addon name gets its own separate dict within this dict
+        self.addon_ctx: Dict[str, Dict[str, Any]] = collections.defaultdict(dict)
         self.session_manager: SessionManager = session_manager or None
         self.selected: SelectionModel = SelectionModel()
         self.regions: List[ProxiedRegion] = []
@@ -50,6 +54,7 @@ class Session(BaseClientSession):
         self.http_message_handler: MessageHandler[HippoHTTPFlow, str] = MessageHandler()
         self.objects = ProxyWorldObjectManager(self, session_manager.settings, session_manager.name_cache)
         self.inventory = ProxyInventoryManager(proxify(self))
+        self.leap_client: Optional[LEAPClient] = None
         # Base path of a newview type cache directory for this session
         self.cache_dir: Optional[str] = None
         self._main_region = None
@@ -185,8 +190,9 @@ class SessionManager:
         self.flow_context = HTTPFlowContext()
         self.asset_repo = HTTPAssetRepo()
         self.message_logger: Optional[BaseMessageLogger] = None
-        self.addon_ctx: Dict[str, Any] = {}
+        self.addon_ctx: Dict[str, Dict[str, Any]] = collections.defaultdict(dict)
         self.name_cache = ProxyNameCache()
+        self.pending_leap_clients: List[LEAPClient] = []
 
     def create_session(self, login_data) -> Session:
         session = Session.from_login_data(login_data, self)
@@ -195,6 +201,15 @@ class SessionManager:
             session.http_message_handler,
         )
         self.sessions.append(session)
+        # TODO: less crap way of tying a LEAP client to a session
+        while self.pending_leap_clients:
+            leap_client = self.pending_leap_clients.pop(-1)
+            # Client may have gone bad since it connected
+            if not leap_client.connected:
+                continue
+            logging.info("Assigned LEAP client to session")
+            session.leap_client = leap_client
+            break
         logging.info("Created %r" % session)
         return session
 
@@ -209,6 +224,8 @@ class SessionManager:
     def close_session(self, session: Session):
         logging.info("Closed %r" % session)
         session.objects.clear()
+        if session.leap_client:
+            session.leap_client.disconnect()
         self.sessions.remove(session)
 
     def resolve_cap(self, url: str) -> Optional["CapData"]:
@@ -218,6 +235,10 @@ class SessionManager:
                 return cap_data
         return CapData()
 
+    async def leap_client_connected(self, leap_client: LEAPClient):
+        self.pending_leap_clients.append(leap_client)
+        AddonManager.handle_leap_client_added(self, leap_client)
+
 
 @dataclasses.dataclass
 class SelectionModel:
